@@ -150,9 +150,9 @@ class ModelFactory():
                             }
                         '''
 
-        self.eigen_paths = self.find_eigen_path()
-        print('eigen instillation found...')
-
+        #self.eigen_paths = self.find_eigen_path()
+        #print('eigen instillation found...')
+        self.eigen_path=''
         self.reserved_model_names = ['build', 'model_maker_cpp', 'models',
                                      'rsnapsim_model_maker']
         self.find_models()
